<?php

namespace Drupal\changelog_preview\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * This class is creating the settings formular for the changelog management.
 *
 * @package Drupal\changelog\Form
 */
class ChangelogManageForm extends ConfigFormBase {

  /**
   * Returns a unique string identifying the form.
   *
   * The returned ID should be a unique string that can be a valid PHP function
   * name, since it's used in hook implementation names such as
   * hook_form_FORM_ID_alter().
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'changelog_manage_form';
  }

  /**
   * Form constructor.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    parent::buildForm($form, $form_state);

    $config = $this->config('changelog_preview.changelog_manage_form');

    $changelog_items_count = $config->get('changelog_items_count') ?? 1;
    $form_state->set('changelog_items_count', $changelog_items_count);

    $form['changelog_items'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Changelog items'),
    ];

    for ($i = 0; $i < $changelog_items_count; $i++) {
      $form['changelog_items']['changelog_item'][$i] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Changelog item (@number)', ['@number' => $i + 1]),
      ];

      $form['changelog_items']['changelog_item'][$i]['changelog_file_path_' . $i] = [
        '#type' => 'textfield',
        '#title' => $this->t('Changelog file path (@number)', ['@number' => $i + 1]),
        '#default_value' => $config->get('changelog_file_path_' . $i),
      ];

      $form['changelog_items']['changelog_item'][$i]['changelog_name_' . $i] = [
        '#type' => 'textfield',
        '#title' => $this->t('Changelog name (@number)', ['@number' => $i + 1]),
        '#default_value' => $config->get('changelog_name_' . $i),
      ];

      $form['changelog_items']['changelog_item'][$i]['changelog_browser_path_' . $i] = [
        '#type' => 'textfield',
        '#title' => $this->t('Changelog browser path (@number)', ['@number' => $i + 1]),
        '#default_value' => $config->get('changelog_browser_path_' . $i),
      ];
    }

    // Add button to add more fields.
    $form['changelog_items']['add_field'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add another changelog'),
      '#submit' => ['::addField'],
      '#ajax' => [
        'callback' => '::updateForm',
        'wrapper' => 'input-fields-wrapper',
      ],
    ];

    $form['#prefix'] = '<div id="input-fields-wrapper">';
    $form['#suffix'] = '</div>';

    // Group submit handlers in an actions element with a key of "actions" so
    // that it gets styled correctly, and so that other modules may add actions
    // to the form. This is not required, but is convention.
    $form['actions'] = [
      '#type' => 'actions',
    ];

    // Add a submit button that handles the submission of the form.
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];
    return $form;
  }

  /**
   * Submit handler for the "Add another field" button.
   */
  public function addField(array &$form, FormStateInterface $form_state) {
    $changelog_items_count = $form_state->get('changelog_items_count');

    $config = $this->config('changelog_preview.changelog_manage_form');
    $config->set('changelog_items_count', $changelog_items_count + 1);
    $config->save();

    $form_state->set('changelog_items_count', $changelog_items_count + 1);
    $form_state->setRebuild();
  }

  /**
   * Ajax callback to update the form.
   */
  public function updateForm(array &$form, FormStateInterface $form_state) {
    return $form;
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $changelog_items_count = $form_state->get('changelog_items_count');
    $config = $this->config('changelog_preview.changelog_manage_form');

    for ($i = 0; $i < $changelog_items_count; $i++) {
      $config->set('changelog_file_path_' . $i, $form_state->getValue('changelog_file_path_' . $i));
      $config->set('changelog_name_' . $i, $form_state->getValue('changelog_name_' . $i));
      $config->set('changelog_browser_path_' . $i, $form_state->getValue('changelog_browser_path_' . $i));
    }

    $config->set('changelog_items_count', $changelog_items_count);

    $config->save();

    $messenger = \Drupal::messenger();
    $messenger->addMessage('Updated path to changelog.');
  }

  /**
   * Gets the configuration names that will be editable.
   *
   * @return array
   *   An array of configuration object names that are editable if called in
   *   conjunction with the trait's config() method.
   */
  protected function getEditableConfigNames() {
    return ['changelog_preview.changelog_manage_form'];
  }

}
